#include "colour.h"

colour colour::correct_gamma(int samples)
{
    double r = x();
    double g = y();
    double b = z();

    // divide the colour total by the number of samples and gamma-correct for gamma=2.0
    auto scale = 1.0 / samples;
    r = sqrt(scale * r);
    g = sqrt(scale * g);
    b = sqrt(scale * b);

    return colour(r, g, b);
}

int format_component(double component)
{
    return int(256 * math::clamp(component, 0.0, 0.999));
}

void write_colour_to_socket(int sockfd, colour pixel_colour, int samples_per_pixel)
{
    pixel_colour = pixel_colour.correct_gamma(samples_per_pixel);

    int r = format_component(pixel_colour.x());
    int g = format_component(pixel_colour.y());
    int b = format_component(pixel_colour.z());

    // pack values
    int len = 3;
    char s[len];
    sprintf(s, "%c%c%c", r, g, b);

    int written = write(sockfd, s, len);
    if (written < 0)
    {
        error("ERROR writing colour to socket");
    }
}

void write_colour_to_stream(std::ostream &out, colour pixel_colour, int samples_per_pixel)
{
    pixel_colour = pixel_colour.correct_gamma(samples_per_pixel);

    auto r = pixel_colour.x();
    auto g = pixel_colour.y();
    auto b = pixel_colour.z();
    
    // write the translated [0,255] value of each colour component.
    out << format_component(r) << ' '
        << format_component(g) << ' '
        << format_component(b) << '\n';
}

colour operator*(const colour& u, const colour& v)
{
    vec3 value = (vec3)u * (vec3)v;
    return colour(value.x(), value.y(), value.z());
}

colour lerp(const colour& a, const colour& b, double t)
{
    vec3 value = lerp((vec3)a, (vec3)b, t);
    return colour(value.x(), value.y(), value.z());
}
