#include "world.h"

colour trace(const world& world, const ray& r, int depth)
{
    hit_record rec;
    if (depth <= 0)
    {
        return grey;
    }

    if (world.hit(r, 0.001, math::infinity, rec))
    {
        ray     scattered;
        colour  attenuation;

        if (rec.mat_ptr->scatter(r, rec, attenuation, scattered))
        {
            return attenuation * trace(world, scattered, depth-1);
        }

        return grey;
    }

    vec3 unit_direction = normalize(r.direction());
    auto t = 0.5 * (unit_direction.y() + 1.0) + 0.5;

    return lerp(grey, pink, t);
}

world* world::close_glass_sphere()
{
    world* w = new world();

    auto material1 = std::make_shared<dielectric>(1.5);
    w->add(std::make_shared<sphere>(point3(0, 0, 0), 3.0, material1));

    //auto material2 = make_shared<lambertian>(pink);
    //world.add(make_shared<sphere>(point3(-4, 1, 0), 1.0, material2));
    
    auto material3 = std::make_shared<metal>(pink, 0.5);
    int sphere_count = 10;
    for (int i = 0; i < sphere_count; i++)
    {
        float a = 6.28 * (float)i/sphere_count - 100.0;
        float r = 8.0;
        float x = r*sin(a);
        float y = 2.0*cos(a);
        float z = r*cos(a);
        point3 pos(x,y,z);
        w->add(std::make_shared<sphere>(pos, 2.0, material3));
    }

    return w;
}
